--
-- Sonic Adventure-esque monitor display
-- (c) K.S. 2014
-- (Extremely minor tweaks to update it to 2.2 by TehRealSalt)
-- Permission is granted to use this code in your own mods.
-- This code should be future-proof and work with all custom monitors.
--

local MYSTERYBOX_DETECT = 296396

--
-- HUD Code
--
local function draw_monitor(v, stplyr)
	if not stplyr.monitor_list or stplyr.monitor_time <= 0
		return
	end

	local defaulty = 185

	-- Slowly scale down the icons when time is nearly up.
	local scale = FRACUNIT
	if stplyr.monitor_time < TICRATE
		scale = stplyr.monitor_time<<FRACBITS / TICRATE
	end

	for i, monitor in ipairs(stplyr.monitor_list)
		-- Get the sprite name and cache it
		local sprite = v.getSpritePatch(monitor.spr, monitor.fr, 0)

		-- in case there's more overlay monitors in the future!
		local overlay = {
			spr = nil,
			x = 0, y = 0,
			color = SKINCOLOR_NONE,
			tc = TC_DEFAULT,
		}

		if (monitor.spr == SPR_TV1U)
			if (stplyr.mo and stplyr.mo.valid)
			and (P_IsValidSprite2(stplyr.mo, SPR2_LIFE))
				sprite = v.getSpritePatch(SPR_TV1P, monitor.fr, 0)
				overlay.spr = v.getSprite2Patch(stplyr.mo.skin, SPR2_LIFE, (stplyr.powers[pw_super] > 0), 0, 0)
				overlay.y = -4
				overlay.color = stplyr.mo.color --stplyr.skincolor
				overlay.tc = stplyr.mo.skin
			end
		end

		local mtc = TC_DEFAULT
		if (monitor.colorized == true)
			mtc = TC_RAINBOW
		end
		local mcm = v.getColormap(mtc, monitor.color)
		v.drawScaled(
			monitor.x<<FRACBITS,
			defaulty<<FRACBITS,
			scale, sprite, V_SNAPTOBOTTOM|V_HUDTRANS|V_PERPLAYER, mcm
		)

		if (overlay.spr != nil)
			local cm = v.getColormap(overlay.tc, overlay.color)
			v.drawScaled(
				(monitor.x<<FRACBITS) + (overlay.x * scale),
				(defaulty<<FRACBITS) + (overlay.y * scale),
				scale, overlay.spr, V_SNAPTOBOTTOM|V_HUDTRANS|V_PERPLAYER, cm
			)
		end
	end
end

customhud.SetupItem("monitordisplay", "sa1hud", draw_monitor, "game", INT32_MIN);

local function insert_monitor(player, mobj, sstate, mysteryscale)
	-- Question box hack
	-- This is unfortunately stupid, but required.
	-- MonitorPop doesn't give us any info about what was spawned.
	if (sstate == MYSTERYBOX_DETECT)
	and (mysteryscale != nil)
		for mobj in mobjs.iterate()
			if mobj.destscale != MYSTERYBOX_DETECT
				continue
			end
			-- Remains
			mobj.destscale = mysteryscale
			if not (mobj.flags & MF_BOXICON)
				continue
			end
			sstate = states[mobj.info.spawnstate]
		end
	end

	local sprite = sstate.sprite
	local frame = (sstate.frame & FF_FRAMEMASK)

	-- Init if it doesn't exist
	if (player.monitor_list == nil)
		player.monitor_list = {}
	end

	-- Remove oldest if we're out of room
	if (#player.monitor_list > 8)
		table.remove(player.monitor_list, 1)
	end

	-- And now insert our new monitor into the table!
	table.insert(player.monitor_list, {x = -12, spr = sprite, fr = frame, color = mobj.color, colorized = mobj.colorized})
	player.monitor_time = TICRATE*5/2
end

--
-- MonitorPop drop-in replacement
--
function A_MonitorPop(actor, var1, var2)
	if not actor.target or not actor.target.player
		-- Make sure to call the super function!
		super(actor, var1, var2)
		return
	end

	-- This is a bit fun.
	-- Get the target player and sprite name
	-- before MonitorPop is done, since they're not valid
	-- after it's called.
	local player = actor.target.player
	local sstate = states[mobjinfo[actor.info.damage].spawnstate]

	-- Question box hack
	local realscale = actor.destscale
	if (actor.type == MT_MYSTERY_BOX)
		sstate = MYSTERYBOX_DETECT
		actor.destscale = MYSTERYBOX_DETECT
	end

	-- Call old monitor pop
	super(actor, var1, var2)

	insert_monitor(player, actor, sstate, realscale)
end

function A_GoldMonitorPop(actor, var1, var2)
	if not actor.target or not actor.target.player
		-- Make sure to call the super function!
		super(actor, var1, var2)
		return
	end

	-- This is a bit fun.
	-- Get the target player and sprite name
	-- before MonitorPop is done, since they're not valid
	-- after it's called.
	local player = actor.target.player
	local sstate = states[mobjinfo[actor.info.damage].spawnstate]

	-- Call old monitor pop
	super(actor, var1, var2)

	insert_monitor(player, actor, sstate, nil)
end

--
-- Handling player's monitors
-- (HUD code can do no modifying, so)
--
local function handle_monitors(player)
	-- Initialization if not already set
	if player.monitor_time == nil
		player.monitor_time = 0
	end

	-- Decrement monitor time if it is set.
	-- If not then just remove the list of monitors.
	if player.monitor_list and player.monitor_time > 0
		player.monitor_time = $1 - 1
	else
		player.monitor_list = nil
		return
	end

	-- And for any monitor pairs we have..
	for i, monitor in ipairs(player.monitor_list)
		-- Get the final X position first;
		-- the final position should have all elements perfectly center
		local finalxpos = 160 + ((#player.monitor_list-1) * 16) - ((i-1) * 32)

		-- Now move the monitor icon on screen ... at least 8 pixels
		monitor.x = $1 + max(8, (finalxpos - monitor.x) / 6)

		-- And if it's past the destination, just put it back there.
		if monitor.x > finalxpos
			monitor.x = finalxpos
		end
	end
end

addHook("PlayerThink", handle_monitors)
